/*
 * tvprogxml
 * Copyright (c) 2003       Erik Waling <erikw@acc.umu.se>
 * Distributed under the terms of the GNU GPL. See the COPYING file
 * for a copy of the GNU Genereal Public License.
 */

#include "main.h"

void cfg_read(struct config *c) {
    
    FILE   *fp;
    struct stat file_stat;
    char   *buf, *value;

    buf = (char *) malloc(BUFSIZE);
    
    sprintf(buf, "%s/%s", getenv("HOME"), CONFIG_DIR);

    if ((stat(buf, &file_stat)) == -1) {
        if (errno == ENOENT) {
            printf("Creating directory: %s\n", buf);
            mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR);
        }
    }

    sprintf(buf, "%s/%s/%s", getenv("HOME"), CONFIG_DIR, CONFIG_FILE);
    
    if ((fp = fopen(buf,"r")) == NULL) {
        printf("Unable to open %s\n",buf); /* FIXME: create config */
        printf("You need to create a config. Check the README and example config.\n");
        exit(EXIT_FAILURE); 
    }   
    
    while ((fgets(buf, BUFSIZE-1, fp)) != NULL) {
        value = strchr(buf, '=');
        if (value != NULL) {
            *value = '\0';
            value++;
            value[strlen(value)-1] = '\0'; /* remove \n */
            /* now buf contains is a pointer to the option
             * and value is a pointer to that options value */

            if (cfg_compare(buf, "external_alarm"))
                cfg_set_str(&c->extalarm, value);
            else if (cfg_compare(buf, "alarm_minutes"))
                cfg_set_int(&c->alarm_minutes, value);
            else if (cfg_compare(buf, "xmltv_file")) {  
                cfg_set_str(&c->xmltv_file, value);
                replace_str_token(getenv("HOME"), "~", c->xmltv_file, 0);
            }
            else if (cfg_compare(buf, "reload_hour"))
                cfg_set_int(&c->reload_hour, value);
            else if (cfg_compare(buf, "channelname_chars"))
                cfg_set_int(&c->channelname_chars, value);
            else if (cfg_compare(buf, "show_next_program"))
                cfg_set_bool(&c->show_next, value);
            else if (cfg_compare(buf, "current_percentage"))
                cfg_set_int(&c->curr_percentage, value);
            else if (cfg_compare(buf, "free_after_hours"))
                cfg_set_int(&c->free_after_hours, value);
            else if (cfg_compare(buf, "color"))
                cfg_set_bool(&c->color, value);
            else if (cfg_compare(buf, "channel_color"))
                cfg_set_color(&c->channel_color, value);
            else if (cfg_compare(buf, "time_color"))
                cfg_set_color(&c->time_color, value);
            else if (cfg_compare(buf, "current_color"))
                cfg_set_color(&c->current_color, value);
            else if (cfg_compare(buf, "next_color"))
                cfg_set_color(&c->next_color, value);
            else if (cfg_compare(buf, "menu_border_color"))
                cfg_set_color(&c->menu_border_color, value);
            else if (cfg_compare(buf, "menu_text_color"))
                cfg_set_color(&c->menu_text_color, value);
            
        }
        memset(buf, 0, BUFSIZE);
    }

    fclose(fp);
    
    
    
    free(buf);
}

void cfg_read_sortorder(struct channel *ch) {

    FILE   *fp;
    char   *buf;
    char count=1;
    
    buf = (char *) malloc(BUFSIZE);
    
    sprintf(buf, "%s/%s/%s", getenv("HOME"), CONFIG_DIR, CHANNEL_ORDER);
    
    if ((fp = fopen(buf,"r")) == NULL) {
        return;
    }   
        
    while ((fgets(buf, BUFSIZE-1, fp)) != NULL) {
        buf[strlen(buf)-1] = '\0';
        if (cfg_add_placement(ch, buf, count))
            count++;   
        memset(buf, 0, BUFSIZE);
    }
    
    fclose(fp);
    free(buf);
/*    
    while (ch) {
        printf("Channel: %s     Order: %d\n",ch->channel_name, ch->placement);
        ch = ch->next;
    }
*/    
}

char cfg_add_placement(struct channel *ch, char *str, char count) {
    
    if ((strlen(str)) == 0)
        return 0;
    
    while (ch) {
        if ((strcasecmp(ch->channel_name, str)) == 0) {
            ch->placement = count;
            return 1;
        }
        ch = ch->next;
    }
    return 0;
}

void cfg_read_watch(struct config *c) {
    
    FILE   *fp;
    char   *buf, *value;
    char   d, t;
    int    i;
    buf = (char *) malloc(BUFSIZE);
    
    sprintf(buf, "%s/%s", getenv("HOME"), CONFIG_DIR);

    sprintf(buf, "%s/%s/%s", getenv("HOME"), CONFIG_DIR, WATCH_FILE);
    
    if ((fp = fopen(buf,"r")) == NULL) {
        return;
    }   
    
    while ((fgets(buf, BUFSIZE-1, fp)) != NULL) {
        value = strchr(buf, ' ');
        if (value != NULL) {
            *value = '\0';
            value++;
            value[strlen(value)-1] = '\0'; /* remove \n */
            
            d = 0;
            t = 0;
            
            for (i=0; *(buf+i) != '\0'; i++) {
                if ( *(buf+i) == 't' || *(buf+i) == 'T' )
                    t = 1;
                if ( *(buf+i) == 'd' || *(buf+i) == 'D' )
                    d = 1;
            }
            
            if (t)
                c->watch = cfg_add_watch(c->watch, value, 1);
            if (d)
                c->watch = cfg_add_watch(c->watch, value, 2);
            
        }
        memset(buf, 0, BUFSIZE);
    }
    fclose(fp);
    free(buf);
}



char cfg_compare(char *buf, char *str) {

    if ((strcasecmp(buf, str)) == 0)
        return 1;

    return 0;
}

void cfg_set_str(char **cfg, char *value) {

    if (*cfg)
        free(*cfg);

    *cfg = strdup(value);
}

void cfg_set_int(int *cfg, char *value) {

    *cfg = atoi(value);
}

void cfg_set_color(char *cfg, char *value) {
    
    if (cfg_compare("black", value))
        *cfg = COLOR_BLACK;
    else if (cfg_compare("red", value))
        *cfg = COLOR_RED;
    else if (cfg_compare("green", value))
        *cfg = COLOR_GREEN;
    else if (cfg_compare("yellow", value))
        *cfg = COLOR_YELLOW;
    else if (cfg_compare("blue", value))
        *cfg = COLOR_BLUE;
    else if (cfg_compare("magenta", value))
        *cfg = COLOR_MAGENTA;
    else if (cfg_compare("cyan", value))
        *cfg = COLOR_CYAN;
    else if (cfg_compare("white", value))
        *cfg = COLOR_WHITE;
}
    
void cfg_set_bool(char *cfg, char *value) {
    
    if (cfg_compare("TRUE", value))
        *cfg = 1;
    else
        *cfg = 0;
}

struct watches *cfg_add_watch(struct watches *w, char *str, char type) {
    
    struct watches *wp = w;    

    if (w) { 
        /* check if watch regexp already exists in lists */
        while (w) {
            if ((strcmp(w->regexp, str)) == 0) {
                if (w->type == type)
                    return wp;
            }
            w = w->next;
        }
        w = wp;        
    }
    /* add watch */
    if (w) {
        while (w->next) {
            w = w->next;
        }
        w->next = (struct watches *) malloc(sizeof(struct watches));
        memset(w->next, 0, sizeof(struct watches));
        w = w->next;
        if (str)
            w->regexp = strdup(str);
        w->type = type;
        
        return wp; 
    }
    else {
        w = (struct watches *) malloc(sizeof(struct watches));
        memset(w, 0, sizeof(struct watches));
        if (str)
            w->regexp = strdup(str);
        w->type = type;
        
        return w;
    }
    
}

char *replace_str_token(char *with, char *replace, char *str, size_t start) { 
 
    /* Replace every occurance of string pointed to by 'replace'
     * with the string pointed to by 'with'.
     * Call with start == 0
     */
      
    char *new, *p, *q;
    int i;
    size_t t;
    
    if ((p = strstr(str+start, replace)) == NULL) 
        return str;

    new = (char *) malloc(strlen(str) + strlen(with) - strlen(replace)+1);
    memset(new, 0, sizeof(new));
    
    memcpy(new, str, p-str);
    
    memcpy(new + (p-str), with, strlen(with));

    for (i=0; *(p+i) == *(replace+i); i++)
        q = p+i;
    
    memcpy(new + (p-str) + strlen(with), ++q, strlen(q)); 
    t = q-str; 
    free(str);
    
    if ((strstr(new+t, replace)) != NULL) 
        new = replace_str_token(with, replace, new, t);
    
    return new;
}
    
void cfg_check_config(void) {

    char error = 0;
    
    if (!conf.xmltv_file) {
        printf("Set a path to your 'xmltv_file' in the config.\n");
        error = 1;
    }
        
    if (!((conf.reload_hour >= 0) && (conf.reload_hour <= 23))) {
        printf("Set a correct value on 'reload_hour' in the config. (0-23)\n");
        error = 1;
    }
    if (conf.free_after_hours < 0) {
        printf("Set a correct value on 'free_after_hours' in the config. (>=0)\n");
        error = 1;
    }
    if ((conf.curr_percentage < 10) || (conf.curr_percentage > 100))  {
        printf("Set a correct value on 'current_percentage' in the config. (10-100)\n");
        error = 1;
    }
    if (conf.channelname_chars <= 0) {
        printf("Set a correct value on 'channelname_chars' in the config. (>0)\n");
        error = 1;
    }
    if (conf.alarm_minutes < 0) {
        printf("Set a correct value on 'alarm_minutes' in the config. (>=0)\n");
        error = 1;
    }
    
    if (error) {
        printf("Exiting!\n");
        exit(EXIT_FAILURE);
    }
}
void cfg_set_default(void) {

    memset(&conf, 0, sizeof(conf));
    conf.alarm_minutes = 5;
    conf.show_next = 1;
    conf.color = 1;

    conf.channel_color = COLOR_CYAN;
    conf.time_color = COLOR_RED;
    conf.current_color = COLOR_GREEN;
    conf.next_color = COLOR_BLUE;
    conf.channelname_chars = 15;

    conf.curr_percentage = 50;
    conf.free_after_hours = 12;
}

    
        
