/*
 * tvprogxml
 * Copyright (c) 2003       Erik Waling <erikw@acc.umu.se>
 * Distributed under the terms of the GNU GPL. See the COPYING file
 * for a copy of the GNU Genereal Public License.
 */

#include "main.h"

void init_curses(void) {

    initscr();

    if (conf.color) {
        if (has_colors())
            init_colors();
        else
            conf.color=0;
    }

    signal(SIGWINCH, winresize);
    cbreak();
    noecho();
    printf("\033[?25l");
    halfdelay(10);
    draw_screen();
}

void init_colors(void) {
    
    start_color();
    use_default_colors();

    init_pair(1, conf.channel_color, -1);
    init_pair(2, conf.time_color, -1);
    init_pair(3, conf.current_color, -1);
    init_pair(4, conf.next_color, -1);
    init_pair(5, conf.menu_border_color, -1);
    init_pair(6, conf.menu_text_color, -1);
        
    
}

void kill_curses(void) {
    
    printf("\033[?25h");
    endwin();
}

void winresize(int signum) {

    struct winsize wsiz;
    ioctl(0, TIOCGWINSZ, &wsiz);
    resizeterm(wsiz.ws_row, wsiz.ws_col);
    printf("\033[?25l"); /* aterm turns the cursor back on after resize */
    draw_screen();
}

void draw_screen(void) {
    clear();
    get_channels(chan);
    get_programs(chan);
    refresh();
}


void print_channel_names(char *name, int y) {

    if (conf.color)
        attron(COLOR_PAIR(1));

    mvprintw(y, 1, "%-*.*s", conf.channelname_chars, conf.channelname_chars , name);

}
    
void print_current_programs(char *program, time_t start, int y, char watch) {

    struct tm *tim = localtime(&start);
    float perc = ((((float) stdscr->_maxx - (conf.channelname_chars+2))/100) * (float) conf.curr_percentage);
    /*   
    mvprintw(18, 1, "x: %d", stdscr->_maxx);
    mvprintw(19, 1, "chars: %d", (int) perc);
    mvprintw(20, 1, "f: %f", perc);
    */
    if (conf.color)
        attron(COLOR_PAIR(2));
    
    if (program)
        mvprintw(y, conf.channelname_chars+2, "%.2d:%.2d",tim->tm_hour, tim->tm_min);
    else
        mvprintw(y, conf.channelname_chars+2, "     ");
    
    if (conf.color)
        attron(COLOR_PAIR(3));
    if (watch)
        attron(A_BOLD);

    if (conf.show_next) {
        if (program)
            mvprintw(y, conf.channelname_chars+8, "%-*.*s", (int) perc-6, (int) perc-6, program);
        else
            mvprintw(y, conf.channelname_chars+8, "%-*.*s", (int) perc-6, (int) perc-6, " ");
    }
    else {
        if (program)
            mvprintw(y, conf.channelname_chars+8, "%-*.*s", stdscr->_maxx-conf.channelname_chars-8, stdscr->_maxx-conf.channelname_chars-8, program);
        else
            mvprintw(y, conf.channelname_chars+8, "%-*.*s", stdscr->_maxx-conf.channelname_chars-8, stdscr->_maxx-conf.channelname_chars-8, " ");
    }
    attroff(A_BOLD);
}

void print_next_programs(char *program, time_t start, int y, char watch) {

    struct tm *tim = localtime(&start);
    float perc = ((((float) stdscr->_maxx - (conf.channelname_chars+2))/100) * (float) conf.curr_percentage);
    
    if (conf.color)
        attron(COLOR_PAIR(2));
    
    if (program)
        mvprintw(y, conf.channelname_chars+2 + (int) perc + 1, "%.2d:%.2d", tim->tm_hour, tim->tm_min);
    else
        mvprintw(y, conf.channelname_chars+2 + (int) perc + 1, "     ");
    
    if (conf.color)
        attron(COLOR_PAIR(4));
    if (watch)
        attron(A_BOLD);
    
    if (program)
        mvprintw(y, conf.channelname_chars+2 + (int) perc + 7, "%-*.*s",(stdscr->_maxx - (conf.channelname_chars+2 + (int) perc) - 7) ,(stdscr->_maxx - (conf.channelname_chars+2 + (int) perc) - 7),program);
    else
        mvprintw(y, conf.channelname_chars+2 + (int) perc + 7, "%-*.*s",(stdscr->_maxx - (conf.channelname_chars+2 + (int) perc) - 7) ,(stdscr->_maxx - (conf.channelname_chars+2 + (int) perc) - 7), " ");

    attroff(A_BOLD);
}

void find_program(struct channel *ch) {

    WINDOW *win;
    MENU *menu_list; 
    ITEM **item_list;
    
    char buf[BUFSIZE];
    struct channel *hits = NULL;
    int i=0;
    struct strlist *data = (struct strlist *) malloc(sizeof(struct strlist));
    
    memset(&buf, 0, sizeof(buf));
    
    if ((win = newwin(3, stdscr->_maxx/2, (stdscr->_maxy/2)-1, stdscr->_maxx/4)) != NULL) {
        if (conf.color) {
            wattron(win, COLOR_PAIR(5));
        }
        box(win,0,0);
        if (conf.color) {
            wattron(win, COLOR_PAIR(6));
        }
        echo();
        nonl();

        printf("\033[?25h");
        
        mvwprintw(win, 1, 2,"Find: ");
        wrefresh(win);
        mvwgetnstr(win, 1, 8, &buf[0], (stdscr->_maxx/2)-10);
        
        noecho();
        halfdelay(10);
        printf("\033[?25l");
        
        hits = find_search(buf, ch);
        delwin(win);
        draw_screen();

        if (!hits || strlen(buf) == 0) {
            sprintf(buf, "No hits on: %s", strdup(buf));
            message_win(buf); 
            return;
        }
        
        item_list = build_timesorted_itemlist(hits, data);
        menu_list = new_menu((ITEM **) item_list);
        
        if ((win = newwin(stdscr->_maxy-1, stdscr->_maxx/2, 1, stdscr->_maxx/4)) != NULL) {

            if (conf.color) {
                set_menu_back(menu_list, COLOR_PAIR(6));
                set_menu_fore(menu_list, COLOR_PAIR(6) | A_BOLD);
            }
            keypad(win, TRUE);
            set_menu_win(menu_list, win);
            set_menu_sub(menu_list, derwin(win, stdscr->_maxy-3, (stdscr->_maxx/2)-2, 1, 2));
            set_menu_format(menu_list, stdscr->_maxy-3, 1);
            set_menu_mark(menu_list, "*");
            
            if (conf.color)
                wattron(win,COLOR_PAIR(5));

            box(win,0,0);
            post_menu(menu_list);
            wrefresh(win);
            
            while ((i = wgetch(win)) != 27) {
                switch (i) {
                    case KEY_DOWN:    /* down */
                        menu_driver(menu_list, REQ_DOWN_ITEM);
                        break;
                    case KEY_UP:    /* up */
                        menu_driver(menu_list, REQ_UP_ITEM);
                        break;
                    case KEY_PPAGE:   /* page up */
                        menu_driver(menu_list, REQ_SCR_UPAGE);
                        break;
                    case KEY_NPAGE:   /* page down */
                        menu_driver(menu_list, REQ_SCR_DPAGE);
                        break;

                }
            }

            unpost_menu(menu_list);
            free_menu(menu_list);
            delwin(win);
            
            while (*item_list != NULL) {
                free_item(*item_list);
                item_list++;
            }
            
            free_strlist(data);
            draw_screen();
        }
    }
}

void message_win(char *msg) {

    WINDOW *win;
    int msglen = strlen(msg);

    if ((win = newwin(3, msglen+4, (stdscr->_maxy/2)-1, (stdscr->_maxx/2)-((msglen+4)/2))) != NULL) {
        if (conf.color)
            wattron(win,COLOR_PAIR(5));
        
        box(win,0,0);
        
        if (conf.color)
            wattron(win,COLOR_PAIR(6));

        mvwprintw(win,1,2,"%s",msg);
        wrefresh(win);

        while (wgetch(win) != 27);

        delwin(win);
        draw_screen();
    }
}       
            
